package cn.fw.morax.common.utils;

import com.google.common.eventbus.AsyncEventBus;
import com.google.common.eventbus.EventBus;
import com.google.common.util.concurrent.ThreadFactoryBuilder;

import java.util.concurrent.LinkedBlockingDeque;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

/**
 * EventBus事件总线
 *
 * @author : Devin
 * Date: 2020/10/13
 * Time: 17:17
 * Description: Guava EventBus事件总线
 */
public class EventBusUtil {

    private static volatile EventBus eventBus;
    private static volatile AsyncEventBus asyncEventBus;

    /**
     * 双重锁单例模式
     *
     * @return
     */
    private static AsyncEventBus getAsyncEventBus() {
        if (asyncEventBus == null) {
            synchronized (AsyncEventBus.class) {
                if (asyncEventBus == null) {
                    asyncEventBus = new AsyncEventBus(new ThreadPoolExecutor(
                            2,
                            20,
                            2,
                            TimeUnit.SECONDS,
                            new LinkedBlockingDeque<>(),
                            new ThreadFactoryBuilder().setNameFormat("event-thread-%d").setDaemon(true).build()));
                }
            }
        }
        return asyncEventBus;
    }

    /**
     * 双重锁单例模式
     *
     * @return
     */
    private static EventBus getEventBus() {
        if (eventBus == null) {
            synchronized (EventBus.class) {
                if (eventBus == null) {
                    eventBus = new EventBus();
                }
            }
        }
        return eventBus;
    }

    /**
     * 同步方式注册监听
     *
     * @param listener
     */
    public static void register(Object listener) {
        getEventBus().register(listener);
    }

    /**
     * 同步方式取消监听
     *
     * @param listener
     */
    public static void unregister(Object listener) {
        getEventBus().unregister(listener);
    }

    /**
     * 同步方式发送事件
     *
     * @param event
     */
    public static void post(Object event) {
        getEventBus().post(event);
    }

    /**
     * 异步方式注册监听
     *
     * @param listener
     */
    public static void asyncRegister(Object listener) {
        getAsyncEventBus().register(listener);
    }

    /**
     * 异步方式取消监听
     *
     * @param listener
     */
    public static void asyncUnregister(Object listener) {
        getAsyncEventBus().unregister(listener);
    }

    /**
     * 异步方式发送事件
     *
     * @param event
     */
    public static void asyncPost(Object event) {
        getAsyncEventBus().post(event);
    }
}
