package cn.fw.morax.server.controller.app;


import cn.fw.common.exception.BusinessException;
import cn.fw.common.page.AppPage;
import cn.fw.common.web.annotation.ControllerMethod;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.common.web.auth.annotation.CurrentUser;
import cn.fw.data.base.domain.common.Message;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.dto.eval.EvalGroupIndicatorRankDTO;
import cn.fw.morax.domain.dto.query.EvalPoolQueryDTO;
import cn.fw.morax.domain.enums.EvalScopeEnum;
import cn.fw.morax.domain.vo.SelectorVO;
import cn.fw.morax.domain.vo.eval.*;
import cn.fw.morax.domain.vo.kpi.KpiIndicatorRankVO;
import cn.fw.morax.service.biz.SettingDraftBizService;
import cn.fw.morax.service.biz.eval.*;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.annotation.IgnoreAuth;
import cn.fw.security.auth.client.annotation.IgnoreUserToken;
import cn.fw.security.auth.client.enums.AuthType;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotNull;
import java.time.LocalDate;
import java.time.YearMonth;
import java.time.temporal.ChronoField;
import java.time.temporal.TemporalAdjuster;
import java.time.temporal.TemporalAdjusters;
import java.util.List;
import java.util.Objects;
import java.util.Set;

import static cn.fw.common.businessvalidator.Validator.BV;
import static cn.fw.common.web.util.ResultBuilder.success;

/**
 * 考评池
 *
 * @author jiangchao
 * @since 2022-11-09
 */
@Slf4j
@RequiredArgsConstructor
@Authorization(AuthType.APP)
@Validated
@IgnoreAuth
@RestController
@RequestMapping("/app/eval-pool")
public class EvalPoolController {

    private final EvalPoolService evalPoolService;
    private final EvalGroupPoolService evalGroupPoolService;

    /**
     * 人员考评池列表
     *
     * @param dto
     * @return
     * @ignoreParams currentUser
     */
    @GetMapping("/staffs")
    @ControllerMethod("人员考评池列表")
    public Message<AppPage<EvalUserPoolVO>> queryUserPool(@Validated EvalPoolQueryDTO dto) {
        if (PublicUtil.isEmpty(dto.getMonthly())) {
            dto.setMonthly(YearMonth.now());
        }
        return success(evalPoolService.queryUserPool(dto));
    }

    /**
     * 本月可查看的其他考评池选项
     *
     * @param user      用户id
     * @param userId    用户id [管理层查看人员薪酬详情时需传值]
     * @param monthly 月度 [员工人查看自己的酬详情时需传值]
     * @return 薪酬池选项
     * @ignoreParams user
     */
    @GetMapping("/staff-selector")
    @IgnoreUserToken
    @ControllerMethod("本月可查看的其他考评池选项")
    public Message<Set<SelectorVO>> evalPoolSelector(@CurrentUser LoginAuthBean user, Long userId, @RequestParam(required = false) YearMonth monthly) {
        if (Objects.isNull(monthly)) {
            monthly = YearMonth.now();
        }
        if (Objects.isNull(userId)) {
            userId = user.getUserId();
        }
        return success(evalGroupPoolService.evalPoolSelector(userId, monthly));
    }

    /**
     * 考评详情查询 (查看自己考评详情)
     *
     * @param userId    用户id
     * @param monthly 月度
     * @return 绩效池详情
     * @ignoreParams userId
     */
    @GetMapping("/own-detail")
    @ControllerMethod("考评详情查询")
    public Message<EvalUserPoolVO> evalPoolOwnDetail(@CurrentUser Long userId,
                                                     @RequestParam(required = false) YearMonth monthly) {
        if (Objects.isNull(monthly)) {
            monthly = YearMonth.now();
        }
        return success(evalGroupPoolService.queryUserPoolDetail(userId, monthly));
    }

    /**
     * 考评详情查询 (查看他人考评详情)
     *
     * @param evalPoolId    用户id
     * @return 绩效池详情
     * @ignoreParams userId
     */
    @GetMapping("/staff")
    @ControllerMethod("考评详情查询")
    public Message<EvalUserPoolVO> evalPoolDetail(@NotNull(message = "考评池id不能为空") Long evalPoolId) {
        return success(evalGroupPoolService.queryPoolDetail(evalPoolId));
    }

    /**
     * 员工考评得分详情
     *
     * @param evalPoolId
     * @param evalGroupIndicatorId
     * @return
     */
    @GetMapping("/staff/score-detail")
    @IgnoreAuth
    @ControllerMethod("员工考评得分详情")
    public Message<EvalPoolIndicatorDetailVO> getStaffHitIndicatorDetail(@NotNull(message = "考评池id不能为空") Long evalPoolId,
                                                                    @NotNull(message = "指标id不能为空") Long evalGroupIndicatorId,
                                                                    @NotNull(message = "日期不能为空") @RequestParam("dataDate") LocalDate dataDate) {
        return success(evalGroupPoolService.staffHitIndicatorDetail(evalPoolId, evalGroupIndicatorId, dataDate));
    }

    /**
     * 员工奖惩详情
     *
     * @param evalPoolId
     * @param evalGroupRewardId
     * @return
     */
    @GetMapping("/staff/reward-detail")
    @IgnoreAuth
    @ControllerMethod("员工奖惩详情")
    public Message<EvalPoolRewardDetailVO> getStaffRewardDetail(@NotNull(message = "考评池id不能为空") Long evalPoolId,
                                                                @NotNull(message = "指标id不能为空") Long evalGroupRewardId,
                                                                @NotNull(message = "日期不能为空") @RequestParam("dataDate") LocalDate dataDate) {
        return success(evalGroupPoolService.staffHitRewardDetail(evalPoolId, evalGroupRewardId, dataDate));
    }

    /**
     * 人员考评指标排名
     *
     * @param dto
     * @return
     */
    @GetMapping("/staff-indicator-rank")
    @ControllerMethod("人员考评指标排名")
    public Message<KpiIndicatorRankVO> queryKpiIndicatorRank(@Validated EvalGroupIndicatorRankDTO dto) {
        if (dto.getIsEvalGroupIndicator() && PublicUtil.isEmpty(dto.getEvalGroupIndicatorParamId()) &&
                PublicUtil.isEmpty(dto.getEvalGroupIndicatorPreconditionId())) {
                throw new BusinessException("参数错误");
        }
        if (! dto.getIsEvalGroupIndicator() && PublicUtil.isEmpty(dto.getEvalGroupRewardParamId()) &&
                PublicUtil.isEmpty(dto.getEvalGroupRewardPreconditionId())) {
                throw new BusinessException("参数错误");
        }
        return success(evalGroupPoolService.queryIndicatorRank(dto, EvalScopeEnum.STAFF));
    }

    /**
     * 门店考评指标排名
     *
     * @param dto
     * @return
     */
    @GetMapping("/shop-indicator-rank")
    @ControllerMethod("门店考评指标排名")
    public Message<KpiIndicatorRankVO> queryShopIndicatorRank(@Validated EvalGroupIndicatorRankDTO dto) {
        if (dto.getIsEvalGroupIndicator() && PublicUtil.isEmpty(dto.getEvalGroupIndicatorParamId()) &&
                PublicUtil.isEmpty(dto.getEvalGroupIndicatorPreconditionId())) {
            throw new BusinessException("参数错误");
        }
        if (! dto.getIsEvalGroupIndicator() && PublicUtil.isEmpty(dto.getEvalGroupRewardParamId()) &&
                PublicUtil.isEmpty(dto.getEvalGroupRewardPreconditionId())) {
            throw new BusinessException("参数错误");
        }
        return success(evalGroupPoolService.queryIndicatorRank(dto, EvalScopeEnum.SHOP));
    }

    /**
     *  门店、人员考评条件指标排名
     *
     * @param preconditionId
     * @param dataDate
     * @return
     */
    @GetMapping("/reward-cond-rank")
    @ControllerMethod("门店、人员考评条件指标排名")
    public Message<List<EvalGroupRewardRankLogVO>> queryRewardCondRank(Long preconditionId,
                                                                       @NotNull(message = "日期不能为空") @RequestParam("dataDate") LocalDate dataDate) {
        return success(evalGroupPoolService.getRewardCondRankLogs(preconditionId, dataDate));
    }

    /**
     *  门店、人员考评提成指标排名
     *
     * @param commissionId
     * @param dataDate
     * @return
     */
    @GetMapping("/reward-commission-rank")
    @ControllerMethod("门店、人员考评提成指标排名")
    public Message<List<EvalGroupRewardRankLogVO>> queryRewardCommissionRank(Long commissionId,
                                                                             @NotNull(message = "日期不能为空") @RequestParam("dataDate") LocalDate dataDate) {
        return success(evalGroupPoolService.getRewardCondRankLogs(commissionId, dataDate));
    }




    /**
     * 门店考评池列表
     *
     * @param dto
     * @return
     * @ignoreParams currentUser
     */
    @GetMapping("/shops")
    @ControllerMethod("门店考评池列表")
    public Message<AppPage<EvalShopPoolVO>> queryShopPool(@Validated EvalPoolQueryDTO dto) {
        if (PublicUtil.isEmpty(dto.getMonthly())) {
            dto.setMonthly(YearMonth.now());
        }
        return success(evalPoolService.queryShopPool(dto));
    }

    /**
     * 门店考评详情查询
     *
     * @param evalPoolId    用户id
     * @return 绩效池详情
     * @ignoreParams userId
     */
    @GetMapping("/shop")
    @ControllerMethod("门店考评详情查询")
    public Message<EvalShopPoolVO> queryShopPoolDetail(@NotNull(message = "考评池id不能为空") Long evalPoolId) {
        return success(evalGroupPoolService.queryShopPoolDetail(evalPoolId));
    }

    /**
     * 门店考评得分详情
     *
     * @param evalPoolId
     * @param evalGroupIndicatorId
     * @return
     */
    @GetMapping("/shop/score-detail")
    @IgnoreAuth
    @ControllerMethod("门店考评得分详情")
    public Message<EvalPoolIndicatorDetailVO> getShopHitIndicatorDetail(@NotNull(message = "考评池id不能为空") Long evalPoolId,
                                                                         @NotNull(message = "指标id不能为空") Long evalGroupIndicatorId,
                                                                         @NotNull(message = "日期不能为空") @RequestParam("dataDate") LocalDate dataDate) {
        return success(evalGroupPoolService.shopHitIndicatorDetail(evalPoolId, evalGroupIndicatorId, dataDate));
    }

        /**
     * 门店奖惩详情
     *
     * @param evalPoolId
     * @param evalGroupRewardId
     * @return
     */
    @GetMapping("/shop/reward-detail")
    @IgnoreAuth
    @ControllerMethod("门店奖惩详情")
    public Message<EvalPoolRewardDetailVO> getShopRewardDetail(@NotNull(message = "考评池id不能为空") Long evalPoolId,
                                                                @NotNull(message = "指标id不能为空") Long evalGroupRewardId,
                                                                @NotNull(message = "日期不能为空") @RequestParam("dataDate") LocalDate dataDate) {
        return success(evalGroupPoolService.shopHitRewardDetail(evalPoolId, evalGroupRewardId, dataDate));
    }

}
