package cn.fw.morax.server.controller.erp;


import cn.fw.common.page.AppPage;
import cn.fw.common.web.annotation.ControllerMethod;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.common.web.auth.annotation.CurrentUser;
import cn.fw.data.base.domain.common.Message;
import cn.fw.morax.domain.dto.EvalIndicatorDTO;
import cn.fw.morax.domain.dto.eval.EvalDTO;
import cn.fw.morax.domain.dto.kpi.IndicatorAbleDTO;
import cn.fw.morax.domain.dto.query.EvalGroupQueryDTO;
import cn.fw.morax.domain.dto.query.EvalIndicatorQueryDTO;
import cn.fw.morax.domain.enums.ApplicableTypeEnum;
import cn.fw.morax.domain.vo.eval.CompositeIndicatorVO;
import cn.fw.morax.domain.vo.eval.EvalIndicatorVO;
import cn.fw.morax.domain.vo.eval.EvalVO;
import cn.fw.morax.service.biz.eval.EvalGroupBizService;
import cn.fw.morax.service.biz.eval.EvalGroupDataService;
import cn.fw.morax.service.biz.eval.EvalIndicatorBizService;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.annotation.IgnoreAuth;
import cn.fw.security.auth.client.enums.AuthType;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotNull;

import java.util.List;

import static cn.fw.common.web.util.ResultBuilder.success;

/**
 * 考评指标
 *
 * @author jiangchao
 * @since 2022-11-09
 */
@Slf4j
@RequiredArgsConstructor
@Authorization(AuthType.ERP)
@Validated
@IgnoreAuth
@RestController
@RequestMapping("/erp/eval")
public class EvalConfigController {

    private final EvalGroupBizService evalGroupBizService;
    private final EvalIndicatorBizService evalIndicatorBizService;

    /**
     * 考评指标列表
     *
     * @param dto
     * @return
     */
    @GetMapping("/indicator/page")
    @ControllerMethod("考评指标列表")
    public Message<AppPage<EvalIndicatorVO>> evalIndicatorPage(@Validated EvalIndicatorQueryDTO dto) {
        return success(evalIndicatorBizService.evalIndicatorPage(dto));
    }

    /**
     * 考评指标详情
     *
     * @return
     */
    @GetMapping("/indicator")
    @ControllerMethod("考评指标详情")
    public Message<EvalIndicatorVO> evalIndicatorDetail(@NotNull(message = "考评指标id不能为空") Long id) {
        return success(evalIndicatorBizService.evalIndicatorDetail(id));
    }

    /**
     * 考评指标禁用启用
     *
     * @return
     */
    @PutMapping("/indicator/enable")
    @ControllerMethod("考评指标禁用启用")
    public Message<Void> enableIndicator(@Validated @RequestBody IndicatorAbleDTO dto) {
        evalIndicatorBizService.enableIndicator(dto.getIndicatorId(), dto.getEnable());
        return success();
    }

    /**
     * 保存考评指标
     *
     * @param dto 考评指标
     * @return
     */
    @PostMapping("/indicator")
    @ControllerMethod("保存考评指标")
    public Message<Void> saveEvalIndicator(@Validated @RequestBody EvalIndicatorDTO dto,
                                                                @CurrentUser LoginAuthBean user) {
        evalIndicatorBizService.saveEvalIndicator(dto, user);
        return success();
    }

    /**
     * 保存考评
     *
     * @param dto 考评
     * @return
     */
    @PostMapping("/group/{submit}")
    @ControllerMethod("保存考评")
    public Message<Void> saveEvalGroup(@Validated @RequestBody EvalDTO dto,
                                       @PathVariable("submit") Integer submit,
                                       @CurrentUser LoginAuthBean user) {
        Boolean isSubmit = (submit == 1);
        evalGroupBizService.saveEvalGroup(dto, user, isSubmit);
        return success();
    }

    /**
     * 考评分页
     *
     * @param dto
     * @return
     */
    @GetMapping("/page")
    @ControllerMethod("考评分页")
    public Message<AppPage<EvalVO>> evalPage(@Validated EvalGroupQueryDTO dto) {

        return success(evalGroupBizService.evalPage(dto));
    }

    /**
     * 考评详情
     *
     * @param id
     * @return
     */
    @GetMapping("/group")
    @ControllerMethod("考评详情")
    public Message<EvalVO> evalGroupDetail(@NotNull(message = "考评组id不能为空") Long id) {

        return success(evalGroupBizService.evalDetail(id));
    }

    /**
     * 获取岗位下的指标
     *
     * @param postId 岗位id
     * @return
     */
    @GetMapping("/post/indicator")
    @ControllerMethod("获取岗位下的指标")
    public Message<List<CompositeIndicatorVO>> getPostIndicators(@NotNull(message = "岗位id不能为空") Long postId) {
        return success(evalIndicatorBizService.getPostIndicators(postId, ApplicableTypeEnum.EVAL));
    }

    /**
     * 获取门店的指标
     *
     * @return
     */
    @GetMapping("/shop/indicator")
    @ControllerMethod("获取门店的指标")
    public Message<List<CompositeIndicatorVO>> getShopIndicators() {
        return success(evalIndicatorBizService.getShopIndicators(ApplicableTypeEnum.EVAL));
    }

    /**
     * 禁用考评
     *
     * @param id 绩效组id
     * @return
     */
    @GetMapping("/group-disable")
    @ControllerMethod("禁用考评")
    public Message<Void> disableEval(@NotNull(message = "绩效组id不能为空") Long id,
                                      @CurrentUser LoginAuthBean currentUser) {
        evalGroupBizService.disableEval(id, currentUser);
        return success();
    }

}
