package cn.fw.morax.service.biz.calculator.salary;

import cn.fw.morax.domain.db.salary.*;
import cn.fw.morax.domain.enums.ParamTypeEnum;
import cn.fw.morax.domain.enums.SalaryCalMethodEnum;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.List;
import java.util.Objects;
import java.util.Optional;

/**
 * @author : kurisu
 * @version : 1.0
 * @className : LadderSalaryCalculator
 * @description : 阶梯提成计算器
 * @date : 2022-04-29 14:18
 */
@Component
@Slf4j
public class LadderSalaryCalculator extends SalaryBaseCalculator {
    @Override
    public BigDecimal calculate(SalaryGroupProject project, SalaryGroupUser user) {
        if (Boolean.TRUE.equals(user.getProbationer())) {
            return null;
        }
        final Long userId = user.getUserId();
        final Long salaryGroupProjectId = project.getId();
        List<SalaryGroupProjectSettin> settins = queryProjectSettins(salaryGroupProjectId);
        if (CollectionUtils.isEmpty(settins)) {
            log.error("[{}]薪酬项配置不存在", salaryGroupProjectId);
            return BigDecimal.ZERO;
        }
        List<SalaryGroupProjectPrecondition> preconditionList = queryPrecondition(salaryGroupProjectId);
        boolean examined = examinePrecondition(preconditionList, user);
        if (!examined) {
            return BigDecimal.ZERO;
        }
        List<SalaryGroupProjectParam> ladderParamList = queryProjectParam(salaryGroupProjectId, ParamTypeEnum.LADDER);
        if (CollectionUtils.isEmpty(ladderParamList)) {
            log.error("配置id[{}]的台阶项不存在", salaryGroupProjectId);
            return BigDecimal.ZERO;
        }
        final BigDecimal ladderValue = calculateParamValue(ladderParamList, user);
        SalaryGroupProjectSettin projectSettin = settins.stream()
                .filter(r -> ladderValue.compareTo(r.getStairMin()) >= 0)
                .filter(r -> ladderValue.compareTo(r.getStairMax()) < 0)
                .findFirst()
                .orElse(null);
        if (Objects.isNull(projectSettin)) {
            log.error("用户[{}]指标配置id[{}]为匹配到任何台阶", userId, salaryGroupProjectId);
            return BigDecimal.ZERO;
        }
        BigDecimal stairValue = projectSettin.getStairValue();
        BigDecimal commissionValue = null;
        List<SalaryGroupProjectParam> projectParamList = queryProjectParam(salaryGroupProjectId, ParamTypeEnum.COMMISSION);
        if (CollectionUtils.isEmpty(projectParamList)) {
            log.info("指标配置id[{}]的提成计算项不存在", salaryGroupProjectId);
            boolean hasWorkAge = ladderParamList.stream().anyMatch(r -> getWorkAgeCode().equals(r.getIndicatorCode()));
            if (hasWorkAge) {
                JSONObject jsonObject = queryUserIndicatorValue(user.getUserId(), user.getDataDate(), getWorkAgeCode());
                stairValue = stairValue.multiply(Optional.ofNullable(jsonObject.getBigDecimal(getWorkDayProportion())).orElse(BigDecimal.ZERO));
            }
        } else {
            commissionValue = calculateParamValue(projectParamList, user);
            stairValue = stairValue.multiply(commissionValue).divide(BigDecimal.ONE, 2, RoundingMode.HALF_UP);
        }
        final BigDecimal _stairValue = stairValue;
        final BigDecimal _commissionValue = commissionValue;
        clearProjectHitLog(salaryGroupProjectId, user);
        saveProjectHitLog(salaryGroupProjectId, user, log -> {
            log.setSalaryGroupProjectSettinId(projectSettin.getId());
            log.setSalaryValue(_stairValue);
            log.setHitLadderValue(ladderValue);
            log.setHitCommissionValue(_commissionValue);
        });
        return stairValue;
    }

    @Override
    public SalaryCalMethodEnum getCalMethod() {
        return SalaryCalMethodEnum.LADDER;
    }
}
