package cn.fw.morax.service.biz.calculator.salary;

import cn.fw.morax.domain.db.kpi.KpiPool;
import cn.fw.morax.domain.db.salary.*;
import cn.fw.morax.domain.enums.ParamTypeEnum;
import cn.fw.morax.domain.enums.SalaryCalMethodEnum;
import cn.fw.morax.domain.enums.StarLevelEnum;
import cn.fw.morax.service.biz.kpi.KpiPoolCommonService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author : kurisu
 * @version : 1.0
 * @className : StarSalaryCalculator
 * @description : 星级薪资计算器
 * @date : 2022-04-27 10:33
 */
@Component
@Slf4j
public class StarSalaryCalculator extends SalaryBaseCalculator {

    private final KpiPoolCommonService kpiPoolCommonService;

    @Autowired
    public StarSalaryCalculator(final KpiPoolCommonService kpiPoolCommonService) {
        this.kpiPoolCommonService = kpiPoolCommonService;
    }

    @Override
    public SalaryCalMethodEnum getCalMethod() {
        return SalaryCalMethodEnum.STAR;
    }

    @Override
    public BigDecimal calculate(SalaryGroupProject param1, SalaryGroupUser param2) {
        if (Boolean.TRUE.equals(param2.getProbationer())) {
            return null;
        }
        final Long userId = param2.getUserId();
        final Long salaryGroupProjectId = param1.getId();
        List<SalaryGroupProjectSettin> settins = queryProjectSettins(salaryGroupProjectId);
        if (CollectionUtils.isEmpty(settins)) {
            log.error("[{}]薪酬项配置不存在", salaryGroupProjectId);
            return BigDecimal.ZERO;
        }
        List<SalaryGroupProjectPrecondition> preconditionList = queryPrecondition(salaryGroupProjectId);
        boolean examined = examinePrecondition(preconditionList, param2);
        if (!examined) {
            return BigDecimal.ZERO;
        }
        //只能查询指定日期月份人员岗位纬度的最新星级
        KpiPool kpiPool = kpiPoolCommonService.queryPool(param2);
        if (Objects.isNull(kpiPool)) {
            log.error("用户[{}]的绩效数据不存在", userId);
            return BigDecimal.ZERO;
        }
        final StarLevelEnum star = kpiPool.getActualStar();
        Map<Long, SalaryGroupProjectSettin> map = settins.stream().collect(Collectors.toMap(SalaryGroupProjectSettin::getStairKey, a -> a));
        SalaryGroupProjectSettin settin = map.get(Long.valueOf(star.getValue()));
        if (Objects.isNull(settin)) {
            log.error("用户[{}]星级[{}]的配置不存在", userId, star.getName());
            return BigDecimal.ZERO;
        }
        BigDecimal stairValue = settin.getStairValue();
        List<SalaryGroupProjectParam> projectParamList = queryProjectParam(salaryGroupProjectId, ParamTypeEnum.COMMISSION);
        if (CollectionUtils.isEmpty(projectParamList)) {
            log.error("配置id[{}]的提成计算项不存在", salaryGroupProjectId);
            return BigDecimal.ZERO;
        }
        final BigDecimal commissionValue = calculateParamValue(projectParamList, param2);
        final BigDecimal salaryValue = stairValue.multiply(commissionValue).divide(BigDecimal.ONE, 4, RoundingMode.HALF_UP);
        clearProjectHitLog(salaryGroupProjectId, param2);
        saveProjectHitLog(salaryGroupProjectId, param2, log -> {
            log.setSalaryGroupProjectSettinId(settin.getId());
            log.setSalaryValue(salaryValue);
            log.setStarLevel(star);
            log.setHitLadderValue(BigDecimal.valueOf(star.getValue()));
            log.setHitCommissionValue(commissionValue);
        });
        return salaryValue;
    }
}
