package cn.fw.morax.service.biz.eval;

import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.common.utils.ThreadPoolUtil;
import cn.fw.morax.domain.bo.eval.EvalGroupCalculableBO;
import cn.fw.morax.domain.bo.eval.EvalGroupUserShop;
import cn.fw.morax.domain.db.eval.*;
import cn.fw.morax.domain.db.salary.SalaryPoolDetail;
import cn.fw.morax.domain.enums.EvalRewardCalMethodEnum;
import cn.fw.morax.domain.enums.EvalScopeEnum;
import cn.fw.morax.domain.enums.ScoreWayEnum;
import cn.fw.morax.service.biz.calculator.eval.kpi.EvalKpiBaseCalculator;
import cn.fw.morax.service.biz.calculator.eval.reward.EvalRewardBaseCalculator;
import cn.fw.morax.service.data.eval.*;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.BoundSetOperations;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDate;
import java.time.YearMonth;
import java.util.*;
import java.util.concurrent.RejectedExecutionException;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.stream.Collectors;

/**
 * @author jiangchao
 * @des: 考评计算
 * @date 2023/1/12 17:39
 */
@Slf4j
@Service
public class EvalCalculateService {

    private final EvalPoolRewardDetailService evalPoolRewardDetailService;
    private final EvalPoolCommonService evalPoolCommonService;
    private final EvalUserPoolService evalUserPoolService;
    private final EvalShopPoolService evalShopPoolService;
    private final EvalGroupService evalGroupService;
    private final EvalGroupIndicatorService evalGroupIndicatorService;
    private final EvalPoolIndicatorDetailService evalPoolIndicatorDetailService;
    private final EvalGroupRewardService evalGroupRewardService;
    private final EvalGroupUserService evalGroupUserService;
    private final StringRedisTemplate stringRedisTemplate;
    private final Map<EvalRewardCalMethodEnum, EvalRewardBaseCalculator> calculatorRewardMap;
    private final Map<ScoreWayEnum, EvalKpiBaseCalculator> calculatorEvalMap;

    public EvalCalculateService(final EvalPoolRewardDetailService evalPoolRewardDetailService,
                                final EvalPoolCommonService evalPoolCommonService,
                                final EvalUserPoolService evalUserPoolService,
                                final EvalShopPoolService evalShopPoolService,
                                final EvalGroupService evalGroupService,
                                final EvalGroupIndicatorService evalGroupIndicatorService,
                                final EvalPoolIndicatorDetailService evalPoolIndicatorDetailService,
                                final EvalGroupRewardService evalGroupRewardService,
                                final EvalGroupUserService evalGroupUserService,
                                final StringRedisTemplate stringRedisTemplate,
                                final List<EvalRewardBaseCalculator> rewardBaseCalculators,
                                final List<EvalKpiBaseCalculator> kpiBaseCalculators) {
        this.evalPoolRewardDetailService = evalPoolRewardDetailService;
        this.evalPoolCommonService = evalPoolCommonService;
        this.evalUserPoolService = evalUserPoolService;
        this.evalShopPoolService = evalShopPoolService;
        this.evalGroupService = evalGroupService;
        this.evalGroupIndicatorService = evalGroupIndicatorService;
        this.evalPoolIndicatorDetailService = evalPoolIndicatorDetailService;
        this.evalGroupRewardService = evalGroupRewardService;
        this.evalGroupUserService = evalGroupUserService;
        this.stringRedisTemplate = stringRedisTemplate;
        this.calculatorRewardMap = rewardBaseCalculators.stream().collect(Collectors.toMap(EvalRewardBaseCalculator::getCalMethod, v -> v));
        this.calculatorEvalMap = kpiBaseCalculators.stream().collect(Collectors.toMap(EvalKpiBaseCalculator::getCalMethod, v -> v));
    }

    @Value("${spring.cache.custom.global-prefix}:calculable:eval-group")
    @Getter
    private String caculEvalGroup;

    /**
     * 缓存需要计算绩效的绩效组信息
     *
     * @param localDate 日期
     */
    public void cacheCalculableEvalGroupIds(final LocalDate localDate) {
        log.info("定时任务【缓存需要计算绩效的绩效组】开始执行");
        Set<Long> evalGroupIds = evalGroupService.queryCalculableEvalGroupIds(localDate);
        if (CollectionUtils.isEmpty(evalGroupIds)) {
            return;
        }
        log.info("缓存需要计算绩效的绩效组信息，kpiGroupIds：{}，日期：{}", JSON.toJSONString(evalGroupIds), localDate);
        cacheGroupIds(evalGroupIds, localDate);
    }


    private void cacheGroupIds(final Set<Long> evalGroupIds, final LocalDate localDate) {
        String[] array = evalGroupIds.stream()
                .map(evalGroupId -> new EvalGroupCalculableBO(evalGroupId, localDate))
                .map(JSONObject::toJSONString)
                .toArray(String[]::new);
        stringRedisTemplate.opsForSet().add(getCaculEvalGroup(), array);
    }


    public void calculateCacheEvalGroup() {
        BoundSetOperations<String, String> setOps = stringRedisTemplate.boundSetOps(getCaculEvalGroup());
        ThreadPoolExecutor threadPool = ThreadPoolUtil.getInstance().getThreadPool();
        List<String> overflowsList = new ArrayList<>();
        String str;
        while ((str = setOps.pop()) != null) {
            final EvalGroupCalculableBO bo = JSONObject.parseObject(str, EvalGroupCalculableBO.class);
            if (Objects.isNull(bo)) {
                continue;
            }
            try {
                String finalStr = str;
                threadPool.execute(() -> {
                    try {
                        calculateEvalGroup(bo);
                    } catch (Exception ex) {
                        log.error("计算考评数据失败：{}", bo, ex);
                        setOps.add(finalStr);
                    }
                });
            } catch (RejectedExecutionException re) {
                overflowsList.add(str);
            }
        }
        if (!CollectionUtils.isEmpty(overflowsList)) {
            for (String s : overflowsList) {
                setOps.add(s);
            }
        }
    }

    @Transactional(rollbackFor = Exception.class)
    public void calculateEvalGroup(EvalGroupCalculableBO bo) {
        log.info("考评组计算：{}", JSON.toJSONString(bo));
        final Long evalGroupId = bo.getEvalGroupId();
        final LocalDate localDate = bo.getLocalDate();

        EvalGroup evalGroup = evalGroupService.getById(evalGroupId);
        if (Objects.isNull(evalGroup)) {
            log.error("考评组[{}]不存在，终止计算！", evalGroupId);
            return;
        }
        List<EvalGroupUserShop> userShops = this.buildEvalGroupObj(evalGroup, localDate, Boolean.TRUE);
        if (CollectionUtils.isEmpty(userShops)) {
            log.error("考评组[{}]人员为空，终止计算！", evalGroupId);
            return;
        }

        List<EvalGroupReward> rewards = evalGroupRewardService.list(Wrappers.<EvalGroupReward>lambdaQuery()
                .eq(EvalGroupReward::getEvalGroupId, evalGroupId)
                .eq(EvalGroupReward::getYn, Boolean.TRUE)
        );
        List<EvalGroupIndicator> indicators = evalGroupIndicatorService.list(Wrappers.<EvalGroupIndicator>lambdaQuery()
                .eq(EvalGroupIndicator::getEvalGroupId, evalGroupId)
                .eq(EvalGroupIndicator::getYn, Boolean.TRUE)
        );

        List<EvalPoolRewardDetail> rewardDetails = new ArrayList<>();
        BigDecimal totalScore = indicators.stream()
                .map(indicator -> BigDecimal.valueOf(indicator.getBaseScore()))
                .reduce(BigDecimal.ZERO, BigDecimal::add);

        //计算
        calcRewardRnkCond(rewards, userShops);
        calcTotalPushMoney(rewards, userShops, rewardDetails);
        for (EvalGroupUserShop userShop : userShops) {
            userShop.setIndicatorTotalScore(totalScore);
            calcRewardMoney(rewards, userShop, rewardDetails);
            calcEvalKpi(userShop, indicators);
        }
        updatePool(evalGroup, localDate, rewardDetails, userShops);
    }

    @Transactional(rollbackFor = Exception.class)
    public void updatePool(EvalGroup evalGroup, LocalDate localDate, List<EvalPoolRewardDetail> rewardDetails, List<EvalGroupUserShop> userShops) {
        final Long evalGroupId = evalGroup.getId();
        EvalScopeEnum evalScope = evalGroup.getScopeType();
        List<Long> poolIds = userShops.stream().map(EvalGroupUserShop::getPoolId).collect(Collectors.toList());
        if (PublicUtil.isNotEmpty(rewardDetails)) {
            evalPoolRewardDetailService.remove(Wrappers.<EvalPoolRewardDetail>lambdaQuery()
                    .in(EvalPoolRewardDetail::getPoolId, poolIds)
                    .eq(EvalPoolRewardDetail::getEvalGroupId, evalGroupId)
                    .eq(EvalPoolRewardDetail::getDataDate, localDate)
            );
            evalPoolRewardDetailService.saveBatch(rewardDetails);
        }
        if (EvalScopeEnum.STAFF.equals(evalScope)) {
            for (EvalGroupUserShop userShop : userShops) {
                BigDecimal scoreRatio = userShop.getScore().divide(userShop.getIndicatorTotalScore(), 2, RoundingMode.DOWN);
                evalUserPoolService.update(Wrappers.<EvalUserPool>lambdaUpdate()
                        .eq(EvalUserPool::getId, userShop.getPoolId())
                        .eq(EvalUserPool::getYn, Boolean.TRUE)
                        .set(EvalUserPool::getReward, userShop.getReward())
                        .set(EvalUserPool::getScore, userShop.getScore())
                        .set(EvalUserPool::getScoreRatio, scoreRatio)
                        .set(EvalUserPool::getMonthly, YearMonth.from(localDate))
                        .set(EvalUserPool::getDataDate, localDate)
                );
            }
        } else {
            for (EvalGroupUserShop userShop : userShops) {
                BigDecimal scoreRatio = userShop.getScore().divide(userShop.getIndicatorTotalScore(), 2, RoundingMode.DOWN);
                evalShopPoolService.update(Wrappers.<EvalShopPool>lambdaUpdate()
                        .eq(EvalShopPool::getId, userShop.getPoolId())
                        .eq(EvalShopPool::getYn, Boolean.TRUE)
                        .set(EvalShopPool::getReward, userShop.getReward())
                        .set(EvalShopPool::getScore, userShop.getScore())
                        .set(EvalShopPool::getScoreRatio, scoreRatio)
                        .set(EvalShopPool::getMonthly, YearMonth.from(localDate))
                        .set(EvalShopPool::getDataDate, localDate)
                );
            }
        }
    }


    public List<EvalGroupUserShop> buildEvalGroupObj(EvalGroup evalGroup, LocalDate localDate, Boolean filterUser) {
        List<EvalGroupUserShop> userShops = new ArrayList<>();
        if (EvalScopeEnum.STAFF.equals(evalGroup.getScopeType())) {
            List<EvalGroupUser> users = evalGroupUserService.list(Wrappers.<EvalGroupUser>lambdaQuery()
                    .eq(EvalGroupUser::getEvalGroupId, evalGroup.getId())
                    .eq(EvalGroupUser::getDataDate, localDate)
//                    .eq(filterUser, EvalGroupUser::getIgnored, Boolean.FALSE)
                    .eq(EvalGroupUser::getYn, Boolean.TRUE)
            );

            Map<Long, EvalUserPool> userPoolMap = evalPoolCommonService.inspectionUserPoolMap(evalGroup, users, localDate);
            for (EvalGroupUser user : users) {
                if (Boolean.TRUE.equals(user.getIgnored())) {
                    continue;
                }
                EvalUserPool userPool = userPoolMap.get(user.getUserId());
                userShops.add(convertBO(user, userPool));
            }
            return userShops;
        }

        List<Long> shopIds = evalGroup.getShopIds();
        Map<Long, EvalShopPool> shopPoolBOMap = evalPoolCommonService.inspectionShopPools(evalGroup, localDate);
        for (Long shopId : shopIds) {
            EvalShopPool shopPool = shopPoolBOMap.get(shopId);
            userShops.add(convertBO(shopPool, evalGroup, localDate));
        }
        return userShops;
    }






    /**
     * 计算绩效组人员的绩效数据
     * 计算单个人员绩效数据
     *
     */
    private void calcEvalKpi(EvalGroupUserShop userShop, List<EvalGroupIndicator> indicators) {
//        KpiPool kpiPool = kpiPoolCommonService.inspectionPool(user);
//        if (Boolean.TRUE.equals(user.getIgnored())) {
//            return;
//        }
        final Long poolId = userShop.getPoolId();
        final Long evalGroupId = userShop.getEvalGroupId();
        List<EvalPoolIndicatorDetail> indicatorDetails = new ArrayList<>();
        for (EvalGroupIndicator indicator : indicators) {
            EvalKpiBaseCalculator calculator = calculatorEvalMap.get(indicator.getScoreWay());
            if (calculator == null) {
                continue;
            }
            BigDecimal score = calculator.calculate(userShop, indicator);
            EvalPoolIndicatorDetail indicatorDetail = createPoolIndicatorValue(score, indicator, userShop.getGroupId());
            indicatorDetail.setDataDate(userShop.getDataDate());
            indicatorDetail.setPoolId(poolId);
            indicatorDetail.setScopeType(userShop.getScopeType());
            indicatorDetails.add(indicatorDetail);
        }

        if (CollectionUtils.isNotEmpty(indicatorDetails)) {
            evalPoolIndicatorDetailService.remove(Wrappers.<EvalPoolIndicatorDetail>lambdaQuery()
                    .eq(EvalPoolIndicatorDetail::getPoolId, poolId)
                    .eq(EvalPoolIndicatorDetail::getEvalGroupId, evalGroupId)
                    .eq(EvalPoolIndicatorDetail::getScopeType, userShop.getScopeType())
                    .eq(EvalPoolIndicatorDetail::getDataDate, userShop.getDataDate())
                    .eq(EvalPoolIndicatorDetail::getYn, Boolean.TRUE));

            evalPoolIndicatorDetailService.saveBatch(indicatorDetails);
        }
        BigDecimal totalScore = BigDecimal.ZERO;
        for (EvalPoolIndicatorDetail indicatorValue : indicatorDetails) {
            totalScore = totalScore.add(indicatorValue.getScore());
        }
        userShop.setScore(totalScore);
    }


    /**
     * 计算考评排名条件
     *
     * @param
     */
    private void calcRewardRnkCond(List<EvalGroupReward> rewards, List<EvalGroupUserShop> userShops) {
        for (EvalGroupReward evalGroupReward : rewards) {
            final String rewardProjectName = evalGroupReward.getName();
            EvalRewardBaseCalculator calculator = calculatorRewardMap.get(evalGroupReward.getCalMethod());
            if (Objects.isNull(calculator)) {
                log.error("[{}_{}]计算器不存在", evalGroupReward.getId(), rewardProjectName);
                return;
            }
            calculator.calculateRankPrecondition(userShops, evalGroupReward);
        }
    }

    /**
     * 计算绩效金额
     *
     * @param rewardDetails
     */
    private void calcTotalPushMoney(List<EvalGroupReward> rewards, List<EvalGroupUserShop> userShops, List<EvalPoolRewardDetail> rewardDetails) {

        List<EvalGroupReward> totalCalcuRewards = rewards.stream()
                .filter(reward -> EvalRewardCalMethodEnum.RANK.equals(reward.getCalMethod()) ||
                        EvalRewardCalMethodEnum.TOTAL_PROPORTION.equals(reward.getCalMethod()))
                .collect(Collectors.toList());

        for (EvalGroupReward evalGroupReward : totalCalcuRewards) {
            final String rewardProjectName = evalGroupReward.getName();
            EvalRewardBaseCalculator calculator = calculatorRewardMap.get(evalGroupReward.getCalMethod());
            if (Objects.isNull(calculator)) {
                log.error("[{}_{}]计算器不存在", evalGroupReward.getId(), rewardProjectName);
                return;
            }
            calculator.calculateGroup(userShops, evalGroupReward);
            for (EvalGroupUserShop userShop : userShops) {
                EvalPoolRewardDetail detail = createDetail(userShop);
                detail.setScopeType(userShop.getScopeType());
                detail.setEvalGroupRewardId(evalGroupReward.getId());
                detail.setAmount(Optional.ofNullable(userShop.getEvalGroupRewardAmount()).orElse(BigDecimal.ZERO));
                rewardDetails.add(detail);
            }
        }
    }
    /**
     * 计算绩效金额
     *
     * @param rewardDetails
     */
    private void calcRewardMoney(List<EvalGroupReward> rewards, EvalGroupUserShop userShop, List<EvalPoolRewardDetail> rewardDetails) {

        List<EvalGroupReward> calcRewards = rewards.stream()
                .filter(reward ->
                        EvalRewardCalMethodEnum.FIXATION.equals(reward.getCalMethod()) ||
                        EvalRewardCalMethodEnum.LADDER.equals(reward.getCalMethod()) ||
                        EvalRewardCalMethodEnum.LADDER_DOUBLE.equals(reward.getCalMethod()))
                .collect(Collectors.toList());
        for (EvalGroupReward evalGroupReward : calcRewards) {
            final String rewardProjectName = evalGroupReward.getName();
            EvalRewardBaseCalculator calculator = calculatorRewardMap.get(evalGroupReward.getCalMethod());
            if (Objects.isNull(calculator)) {
                log.error("[{}_{}]计算器不存在", evalGroupReward.getId(), rewardProjectName);
                return;
            }
            final BigDecimal salaryMoney = calculator.calculate(userShop, evalGroupReward);
            if (Objects.nonNull(salaryMoney)) {
                EvalPoolRewardDetail detail = createDetail(userShop);
                detail.setScopeType(userShop.getScopeType());
                detail.setEvalGroupRewardId(evalGroupReward.getId());
                detail.setAmount(salaryMoney);
                rewardDetails.add(detail);

                userShop.setReward(userShop.getReward().add(salaryMoney));
            }
        }
    }

    private EvalPoolIndicatorDetail createPoolIndicatorValue(BigDecimal score, EvalGroupIndicator indicator, Long groupId) {
        EvalPoolIndicatorDetail indicatorDetail = new EvalPoolIndicatorDetail();
        indicatorDetail.setEvalGroupId(indicator.getEvalGroupId());
        indicatorDetail.setEvalGroupIndicatorId(indicator.getId());
        indicatorDetail.setScore(score);
        indicatorDetail.setYn(Boolean.TRUE);
        indicatorDetail.setGroupId(groupId);
        return indicatorDetail;
    }

    /**
     * 创建实体
     *
     * @return
     */
    private EvalPoolRewardDetail createDetail(EvalGroupUserShop userShop) {
        EvalPoolRewardDetail poolDetail = new EvalPoolRewardDetail();
        poolDetail.setPoolId(userShop.getPoolId());
        poolDetail.setEvalGroupId(userShop.getEvalGroupId());
        poolDetail.setScopeType(userShop.getScopeType());
        poolDetail.setAmount(BigDecimal.ZERO);
        poolDetail.setDataDate(userShop.getDataDate());
        poolDetail.setGroupId(userShop.getGroupId());
        poolDetail.setYn(Boolean.TRUE);
        return poolDetail;
    }

    public EvalGroupUserShop convertBO(EvalGroupUser user, EvalUserPool userPool) {
        EvalGroupUserShop userShop = new EvalGroupUserShop();
        userShop.setScopeType(EvalScopeEnum.STAFF);
        userShop.setEvalGroupId(user.getEvalGroupId());
        userShop.setPoolId(userPool.getId());
        userShop.setReferId(user.getUserId());
        userShop.setDataDate(user.getDataDate());
        userShop.setGroupId(user.getGroupId());
        userShop.setScore(BigDecimal.ZERO);
        userShop.setReward(BigDecimal.ZERO);
        return userShop;
    }

    public EvalGroupUserShop convertBO(EvalShopPool shopPool, EvalGroup evalGroup, LocalDate dataDate) {
        EvalGroupUserShop userShop = new EvalGroupUserShop();
        userShop.setReferId(shopPool.getShopId());
        userShop.setPoolId(shopPool.getId());
        userShop.setScopeType(EvalScopeEnum.SHOP);
        userShop.setEvalId(userShop.getEvalId());
        userShop.setEvalGroupId(evalGroup.getId());
        userShop.setDataDate(dataDate);
        userShop.setGroupId(evalGroup.getGroupId());
        userShop.setScore(BigDecimal.ZERO);
        userShop.setReward(BigDecimal.ZERO);
        return userShop;
    }

}
