package cn.fw.morax.service.biz.eval;

import cn.fw.ehr.sdk.api.enums.StaffStatusEnum;
import cn.fw.morax.common.utils.DateUtil;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.db.eval.EvalGroup;
import cn.fw.morax.domain.db.eval.EvalGroupUser;
import cn.fw.morax.domain.db.kpi.KpiGroup;
import cn.fw.morax.domain.db.kpi.KpiGroupUser;
import cn.fw.morax.domain.db.kpi.KpiStarSpecialRule;
import cn.fw.morax.domain.enums.EvalScopeEnum;
import cn.fw.morax.domain.enums.KpiIgnoreCauseEnum;
import cn.fw.morax.domain.enums.SettingStatusEnum;
import cn.fw.morax.rpc.ehr.EhrRpcService;
import cn.fw.morax.rpc.ehr.dto.PerformanceStaffDTO;
import cn.fw.morax.service.data.eval.EvalGroupService;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.BoundValueOperations;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author jiangchao
 * @des: 考评组人员
 * @date 2023/1/12 14:40
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class EvalGroupUserBizService {

    private final EvalGroupService evalGroupService;
    private final EhrRpcService ehrRpcService;
    private final StringRedisTemplate stringRedisTemplate;

    @Value("${spring.cache.custom.global-prefix}:kpi:anticipated_date:")
    @Getter
    private String startKpiKey;
    @Value("${spring.cache.custom.global-prefix}:kpi:await_job:anticipated_date:")
    @Getter
    private String awaitJobAfterStartKpiKey;

    /**
     * 所有集团生效的绩效组配置
     */
    public Map<Long, List<EvalGroup>> postEffectEvalGroups(){
        LocalDate queryDate = LocalDate.now().minusDays(1L);
        List<EvalGroup> evalGroups = evalGroupService.getAllEffectGroups(queryDate);
        List<EvalGroup> staffEvalGroups = evalGroups.stream()
                .filter(evalGroup -> EvalScopeEnum.STAFF.equals(evalGroup.getScopeType())).collect(Collectors.toList());
        if (PublicUtil.isEmpty(staffEvalGroups)) {
            log.info("时间：{}，没有正在生效中的考评组配置", DateTimeFormatter.ofPattern("yyyy-MM-dd").format(queryDate));
            return Maps.newHashMap();
        }
        return staffEvalGroups.stream().collect(Collectors.groupingBy(EvalGroup::getPostId));
    }

    /**
     * 构建绩效组人员
     *
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public List<EvalGroupUser> fetchBuildKpiUser(Long postId, List<EvalGroup> evalGroups, LocalDate localDate){
        Set<Long> shopIds = new HashSet<>();
        Map<Long, EvalGroup> shopKpiGroupMap = new HashMap<>();
        for (EvalGroup evalGroup : evalGroups) {
            shopIds.addAll(evalGroup.getShopIds());
            for (Long shopId : evalGroup.getShopIds()) {
                shopKpiGroupMap.put(shopId, evalGroup);
            }
        }

        List<PerformanceStaffDTO> staffs = ehrRpcService.queryKpiStaff(postId, new ArrayList<>(shopIds));
        if (PublicUtil.isEmpty(staffs)) {
            return new ArrayList<>();
        }
        List<EvalGroupUser> result = Lists.newArrayListWithCapacity(staffs.size());

        EvalGroup evalGroup = null;
        for (PerformanceStaffDTO staff : staffs) {
            EvalGroupUser evalGroupUser = this.convertToGroupUser(staff, localDate);
            if (! shopKpiGroupMap.containsKey(staff.getShopId())) {
                log.error("员工门店没有绩效组配置，门店：{}，岗位：{}，员工：{}", staff.getShopId(), staff.getPostId(), staff.getId());
                continue;
            }
            evalGroup = shopKpiGroupMap.get(staff.getShopId());
            evalGroupUser.setEvalId(evalGroup.getEvalId());
            evalGroupUser.setEvalGroupId(evalGroup.getId());
            evalGroupUser.setEgc(evalGroup.getEgc());
            //处理绩效考核数据
            this.handleStaffKpi(staff, evalGroupUser);
            result.add(evalGroupUser);
        }
        return result;
    }

    /**
     * 构建绩效组人员
     *
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public List<EvalGroupUser> fetchBuildImportUser(Long postId, List<Long> shopIds, LocalDate localDate){
        List<PerformanceStaffDTO> staffs = ehrRpcService.queryKpiStaff(postId, new ArrayList<>(shopIds));
        if (PublicUtil.isEmpty(staffs)) {
            return new ArrayList<>();
        }
        List<EvalGroupUser> result = Lists.newArrayListWithCapacity(staffs.size());

        for (PerformanceStaffDTO staff : staffs) {
            EvalGroupUser evalGroupUser = this.convertToGroupUser(staff, localDate);
            //处理绩效考核数据
            this.handleStaffKpi(staff, evalGroupUser);
            result.add(evalGroupUser);
        }
        return result;
    }

    /**
     * 处理员工绩效考核数据
     * 试用期、转正两个月内、调岗两个月内、待岗两个月内、申请不进行绩效考核
     *
     * @param staff
     * @param evalUser
     */
    public void handleStaffKpi(PerformanceStaffDTO staff, EvalGroupUser evalUser) {
        LocalDate currentTime = LocalDate.now();
        KpiIgnoreCauseEnum ignoreCauseEnum = null;
        LocalDate startKpiDate = null;
        Long staffId = staff.getId();
        try {
            //试用期不计算绩效  绩效开始时间为试用期转正的两个月后
            if (StaffStatusEnum.PROBATION.getValue().equals(staff.getStaffStatus())) {
                startKpiDate = PublicUtil.getKpiDateOfProbationStaff(PublicUtil.isNotEmpty(staff.getEntryDate()) ? staff.getEntryDate() : LocalDate.now());
                ignoreCauseEnum = KpiIgnoreCauseEnum.PROBATION;
            }

            //转正两个月内不计算绩效  绩效开始时间为转正的两个月后
            if (StaffStatusEnum.REGULAR.getValue().equals(staff.getStaffStatus())) {
                LocalDate regularDate = PublicUtil.isNotEmpty(staff.getRegularDate()) ? staff.getRegularDate() : staff.getEntryDate();
                LocalDate newStartKpiDate = PublicUtil.getKpiDateOfRegularStaff(regularDate);
                if (this.checkNewStartKpiDateValid(startKpiDate, newStartKpiDate, currentTime)) {
                    ignoreCauseEnum = KpiIgnoreCauseEnum.TURN_POSITIVE_AFTER;
                    startKpiDate = newStartKpiDate;
                }
            }

            //绩效组变动不足完整2个月  绩效开始时间为获得岗位的两个月后
            LocalDate postChangeStartKpiDate = this.getStartKpiDate(staffId);
            if (PublicUtil.isNotEmpty(postChangeStartKpiDate)) {
                if (this.checkNewStartKpiDateValid(startKpiDate, postChangeStartKpiDate, currentTime)) {
                    ignoreCauseEnum = KpiIgnoreCauseEnum.TRANSFER_NEW_POSITION_AFTER;
                    startKpiDate = postChangeStartKpiDate;
                }
            }

            //主动申请不计算绩效  绩效开始时间为设置的申请开始绩效考核时间
//            KpiStarSpecialRule specialRule = userRuleMap.get(staffId);
            //开始考核日期
//            if (PublicUtil.isNotEmpty(specialRule) && PublicUtil.isNotEmpty(specialRule.getEarliestStartingMonthly())) {
//                LocalDate newStartKpiDate = specialRule.getEarliestStartingMonthly();
//                //当前时间在开始考核时间之之前    审批通过后，前面月份的绩效不纳入后续计算
//                if (this.checkNewStartKpiDateValid(startKpiDate, newStartKpiDate, currentTime)) {
//                    ignoreCauseEnum = KpiIgnoreCauseEnum.APPLY_NO_ASSESSMENT;
//                    startKpiDate = newStartKpiDate;
//                }
//            }

            //待岗期间
            if (PublicUtil.isNotEmpty(staff.getAwaitJob()) && staff.getAwaitJob()) {
                ignoreCauseEnum = KpiIgnoreCauseEnum.AWAIT_JOB_DURATION;
                //@todo 人事系统待岗中人员没有待岗结束时间
                startKpiDate = null;
            }

            //待岗结束两个月期间
            LocalDate afterAwaitJobStartKpiDate = this.getAfterAwaitJobStartKpiDate(staffId);
            if (PublicUtil.isNotEmpty(afterAwaitJobStartKpiDate) ) {
                if (this.checkNewStartKpiDateValid(startKpiDate, afterAwaitJobStartKpiDate, currentTime)) {
                    ignoreCauseEnum = KpiIgnoreCauseEnum.AWAIT_JOB_AFTER;
                    startKpiDate = afterAwaitJobStartKpiDate;
                }
            }

            if (PublicUtil.isNotEmpty(ignoreCauseEnum)) {
                this.handleIgnoredKpiGroupUser(evalUser, ignoreCauseEnum, startKpiDate);
                return;
            }

            //都未命中上面的条件就计算绩效，开始计算时间为转正两月后、获得新岗位两月后、申请开始考核时间任一
            evalUser.setIgnored(Boolean.FALSE);
        }catch (Exception e) {
            evalUser.setIgnored(Boolean.TRUE);
            log.error("处理绩效组人员数据异常，员工信息：{}", staff, e);
        }
    }

    /**
     * 判断新绩效生效日期 是否有效
     *
     * @param oldStartKpiDate
     * @param newStartKpiDate
     * @return
     */
    private Boolean checkNewStartKpiDateValid(LocalDate oldStartKpiDate, LocalDate newStartKpiDate, LocalDate currentTime) {
        //当前时间在生效日期之后，计算绩效
        if (currentTime.isAfter(newStartKpiDate)) {
            return Boolean.FALSE;
        }
        //旧生效时间为空  或者  新生效日期在旧生效日期之后
        if (PublicUtil.isEmpty(oldStartKpiDate) || newStartKpiDate.isAfter(oldStartKpiDate)) {
            return Boolean.TRUE;
        }
        return Boolean.FALSE;
    }


    /**
     * 处理绩效忽略数据
     *
     * @param evalUser
     * @param ignoreCauseEnum
     * @param anticipatedDate
     */
    private void handleIgnoredKpiGroupUser(EvalGroupUser evalUser, KpiIgnoreCauseEnum ignoreCauseEnum, LocalDate anticipatedDate) {
        evalUser.setIgnored(Boolean.TRUE);
        if (PublicUtil.isNotEmpty(anticipatedDate)) {
            evalUser.setAnticipatedDate(anticipatedDate);
        }
        evalUser.setIgnoreCause(ignoreCauseEnum);
        evalUser.setIgnoreCauseDesc(ignoreCauseEnum.getName());
    }

    /**
     * 检查员工是否在待岗保护期
     *
     * @param userId
     */
    public LocalDate getAfterAwaitJobStartKpiDate(Long userId) {
        String key = getAwaitJobAfterStartKpiKey() + userId;
        if (! stringRedisTemplate.hasKey(key)) {
            return null;
        }
        BoundValueOperations<String, String> strOps = stringRedisTemplate.boundValueOps(key);
        LocalDate startKpiDate = LocalDate.parse(strOps.get());
        LocalDate currentDate = LocalDate.now();
        //当前时间在开始计算绩效时间  之后，已经开始计算绩效
        if (currentDate.isAfter(startKpiDate)) {
            return null;
        }
        return startKpiDate;
    }

    /**
     * 获取开始计算绩效时间
     *
     * @param userId
     */
    public LocalDate getStartKpiDate(Long userId) {
        String key = getStartKpiKey() + userId;
        if (! stringRedisTemplate.hasKey(key)) {
            return null;
        }
        BoundValueOperations<String, String> strOps = stringRedisTemplate.boundValueOps(key);
        LocalDate startKpiDate = LocalDate.parse(strOps.get());
        LocalDate currentDate = LocalDate.now();
        //当前时间在开始计算绩效时间  之后，已经开始计算绩效
        if (currentDate.isAfter(startKpiDate)) {
            return null;
        }
        return startKpiDate;
    }

    /**
     * 转换员工基础信息
     *
     * @param staff
     * @return
     */
    private EvalGroupUser convertToGroupUser(PerformanceStaffDTO staff, LocalDate now){
        EvalGroupUser evalGroupUser = EvalGroupUser.builder()
                .userId(staff.getId())
                .userName(staff.getName())
                .postId(staff.getPostId())
                .postName(staff.getPostName())
                .shopId(staff.getShopId())
                .shopName(staff.getShopName())
                .groupId(staff.getGroupId())
                .dataDate(now)
                .build();
        return evalGroupUser;
    }

}
