package cn.fw.morax.service.biz.kpi;

import cn.fw.morax.common.utils.DateUtil;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.db.kpi.Indicators;
import cn.fw.morax.domain.db.kpi.KpiGroup;
import cn.fw.morax.domain.db.kpi.KpiGroupUser;
import cn.fw.morax.domain.db.kpi.KpiRewardPoint;
import cn.fw.morax.sdk.dto.kpi.KpiGroupUserQuery;
import cn.fw.morax.sdk.dto.kpi.KpiGroupUserResult;
import cn.fw.morax.sdk.dto.kpi.KpiPointRewardReq;
import cn.fw.morax.service.data.kpi.IndicatorsService;
import cn.fw.morax.service.data.kpi.KpiGroupService;
import cn.fw.morax.service.data.kpi.KpiGroupUserService;
import cn.fw.morax.service.data.kpi.KpiRewardPointService;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.YearMonth;
import java.util.*;
import java.util.stream.Collectors;

import static cn.fw.common.businessvalidator.Validator.BV;

/**
 * @author : kurisu
 * @className : KpiReportService
 * @description : 绩效上报相关服务
 * @date : 2022-04-06 15:06
 */

@Slf4j
@RequiredArgsConstructor
@Service
public class KpiReportService {
    private final IndicatorsService indicatorsService;
    private final KpiGroupService kpiGroupService;
    private final KpiGroupUserService kpiGroupUserService;
    private final KpiRewardPointService kpiRewardPointService;
    private final KpiGroupUserBizService kpiGroupUserBizService;


    @Transactional(rollbackFor = Exception.class)
    public void reportKpiReward(KpiPointRewardReq req) {
        LocalDateTime occurredTime = DateUtil.date2LocalDateTime(req.getDataTime());
        KpiRewardPoint kpiRewardPoint = new KpiRewardPoint();
        BeanUtils.copyProperties(req, kpiRewardPoint);
        kpiRewardPoint.setOccurredTime(occurredTime);
        kpiRewardPoint.setMonthly(YearMonth.from(occurredTime));
        if (!Boolean.TRUE.equals(req.getAward())) {
            kpiRewardPoint.setScorePoint(req.getScorePoint().multiply(BigDecimal.valueOf(-1)));
        }
        kpiRewardPointService.save(kpiRewardPoint);
    }


    /**
     * 根据岗位码和绩效指标码查询对应日期的绩效组人员
     *
     * @param kpiGroupUserQuery
     * @return 绩效组人员
     */
    public List<KpiGroupUserResult> getIndicatorUser(final KpiGroupUserQuery kpiGroupUserQuery) {
        final String indicatorCode = kpiGroupUserQuery.getIndicatorCode();
        final Long postId = kpiGroupUserQuery.getPostId();
        final Date dataDate = new Date(kpiGroupUserQuery.getDataDate());
        log.info("查询绩效组人员，日期：{}", new SimpleDateFormat("yyyy-MM-dd").format(dataDate));
        Indicators indicator = indicatorsService.queryByCode(indicatorCode, Boolean.FALSE);
        BV.notNull(indicator, () -> "指标码不存在或已失效");

        ArrayList<KpiGroupUserResult> list = new ArrayList<>();

        List<Long> kpiGroupIdArr = kpiGroupService.queryKgiGroupIdByIndCode(indicatorCode,
                postId, kpiGroupUserQuery.getGroupId(), dataDate);

        if (CollectionUtils.isEmpty(kpiGroupIdArr)) {
            log.error("没有对应的绩效组数据【{}】", kpiGroupUserQuery);
            return list;
        }

        List<KpiGroupUser> userList = kpiGroupUserService.list(Wrappers.<KpiGroupUser>lambdaQuery()
                .in(KpiGroupUser::getKpiGroupId, new HashSet<>(kpiGroupIdArr))
                .eq(KpiGroupUser::getDataDate, DateUtil.date2LocalDate(dataDate))
                .ne(KpiGroupUser::getIgnored, Boolean.TRUE)
                .eq(KpiGroupUser::getYn, Boolean.TRUE)
        );

        if (CollectionUtils.isEmpty(userList)) {
            log.error("没有对应日期的绩效组人员数据【{}】", DateUtil.getStringDateShort(dataDate));
            return list;
        }

        Map<Long, List<KpiGroupUser>> kpiGroupUserMap = userList.stream().collect(Collectors.groupingBy(KpiGroupUser::getKpiGroupId));
//        for (Map.Entry<Long, List<KpiGroupUser>> entry : kpiGroupUserMap.entrySet()) {
//            KpiGroupIndicator groupIndicator = kpiGroupIndicatorService.getOne(Wrappers.<KpiGroupIndicator>lambdaQuery()
//                            .eq(KpiGroupIndicator::getKpiGroupId, entry.getKey())
//                            .eq(KpiGroupIndicator::getIndicatorCode, indicatorCode)
//                            .eq(KpiGroupIndicator::getYn, Boolean.TRUE)
//                    , Boolean.FALSE);
//
//            KpiGroupUserResult result = new KpiGroupUserResult();
//            result.setRoleCodeList(indicator.getRoleCodes());
//            result.setUserIdList(transferUser(entry.getValue()));
//            result.setTargetType(TargetTypeEnum.ofValue(groupIndicator.getTargetType().getValue()));
//            if (!KpiTargetTypeEnum.NO.equals(groupIndicator.getTargetType())) {
//                result.setTargetValue(groupIndicator.getTargetValue());
//            }
//            list.add(result);
//        }

        return list;
    }


    /**
     * 根据岗位码和绩效指标码查询对应日期的绩效组人员
     *
     * @param kpiGroupUserQuery
     * @return 绩效组人员
     */
    public List<KpiGroupUserResult> getRealTimeIndicatorUser(final KpiGroupUserQuery kpiGroupUserQuery) {
        final String indicatorCode = kpiGroupUserQuery.getIndicatorCode();
        final Long postId = kpiGroupUserQuery.getPostId();
        final Date dataDate = new Date();
        final LocalDate localDate = DateUtil.date2LocalDate(dataDate);
        Indicators indicator = indicatorsService.queryByCode(indicatorCode, Boolean.FALSE);
        BV.notNull(indicator, () -> "指标码不存在或已失效");

        ArrayList<KpiGroupUserResult> list = new ArrayList<>();

        List<KpiGroup> kpiGroups = kpiGroupService.queryKgiGroupByIndCode(indicatorCode,
                postId, kpiGroupUserQuery.getGroupId(), dataDate);

        if (CollectionUtils.isEmpty(kpiGroups)) {
            log.error("没有对应的绩效组数据【{}】", kpiGroupUserQuery);
            return list;
        }
        List<Long> kpiGroupIds = kpiGroups.stream().map(KpiGroup::getId).collect(Collectors.toList());
        List<KpiGroupUser> groupUsers = kpiGroupUserService.list(Wrappers.<KpiGroupUser>lambdaQuery()
                .in(KpiGroupUser::getKpiGroupId, new HashSet<>(kpiGroupIds))
                .eq(KpiGroupUser::getDataDate, localDate)
                .ne(KpiGroupUser::getIgnored, Boolean.TRUE)
                .eq(KpiGroupUser::getYn, Boolean.TRUE)
        );

        this.mergeRealTimeUser(postId, kpiGroups, groupUsers, localDate);

        if (CollectionUtils.isEmpty(groupUsers)) {
            log.error("没有对应日期的绩效组人员数据【{}】", DateUtil.getStringDateShort(dataDate));
            return list;
        }

        Map<Long, List<KpiGroupUser>> kpiGroupUserMap = groupUsers.stream().collect(Collectors.groupingBy(KpiGroupUser::getKpiGroupId));
        for (List<KpiGroupUser> valueArr : kpiGroupUserMap.values()) {
            KpiGroupUserResult result = new KpiGroupUserResult();
            result.setRoleCodeList(indicator.getRoleCodes());
            result.setUserIdList(transferUser(valueArr));
            list.add(result);
        }

        return list;
    }

    /**
     * 合并绩效组实时员工
     *
     * @param postId
     * @param kpiGroups
     * @param userList
     * @return
     */
    public void mergeRealTimeUser(Long postId, List<KpiGroup> kpiGroups, List<KpiGroupUser> userList, LocalDate localDate) {
        List<KpiGroupUser> groupUsers = kpiGroupUserBizService.fetchBuildKpiUser(postId, kpiGroups, localDate);
        groupUsers = groupUsers.stream().filter(user -> Boolean.FALSE.equals(user.getIgnored())).collect(Collectors.toList());
        if (PublicUtil.isEmpty(userList)) {
            userList.addAll(groupUsers);
            return;
        }
        Set<Long> userIds = userList.stream().map(KpiGroupUser::getUserId).collect(Collectors.toSet());
        List<KpiGroupUser> notExistUser = groupUsers.stream().filter(user -> (!userIds.contains(user.getUserId()))).collect(Collectors.toList());
        userList.addAll(notExistUser);
    }


    /**
     * 转换器  KpiGroupUser -> SimpleUserData
     *
     * @param userList
     * @return
     */
    private List<KpiGroupUserResult.SimpleUserData> transferUser(List<KpiGroupUser> userList) {
        List<KpiGroupUserResult.SimpleUserData> list = new ArrayList<>();
        for (KpiGroupUser kpiGroupUser : userList) {
            KpiGroupUserResult.SimpleUserData simpleUserData = new KpiGroupUserResult.SimpleUserData();
            simpleUserData.setUserId(kpiGroupUser.getUserId());
            simpleUserData.setUserName(kpiGroupUser.getUserName());
            simpleUserData.setShopId(kpiGroupUser.getShopId());
            simpleUserData.setShopName(kpiGroupUser.getShopName());
            list.add(simpleUserData);
        }

        return list;
    }
}
