package cn.fw.morax.service.biz.salary;

import cn.fw.approval.sdk.param.InitiateApprovalParam;
import cn.fw.common.data.mybatis.pagination.PageData;
import cn.fw.common.exception.BusinessException;
import cn.fw.common.page.AppPage;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.morax.common.config.FlowVal;
import cn.fw.morax.common.pojo.event.ApprovalResultEvent;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.db.ApprovalRecord;
import cn.fw.morax.domain.db.salary.IncomeCompensation;
import cn.fw.morax.domain.db.salary.IncomeCompensationList;
import cn.fw.morax.domain.dto.IncomeCompensationDTO;
import cn.fw.morax.domain.dto.query.IncomeCompensationQueryDTO;
import cn.fw.morax.domain.enums.ApprovalStatusEnum;
import cn.fw.morax.domain.enums.ApprovalTypeEnum;
import cn.fw.morax.domain.enums.SettingStatusEnum;
import cn.fw.morax.domain.vo.salary.IncomeCompensationVO;
import cn.fw.morax.rpc.approval.FlowApproveRpc;
import cn.fw.morax.rpc.approval.dto.FlowDto;
import cn.fw.morax.service.biz.kpi.KpiGroupUserBizService;
import cn.fw.morax.service.data.ApprovalRecordService;
import cn.fw.morax.service.data.salary.IncomeCompensationListService;
import cn.fw.morax.service.data.salary.IncomeCompensationService;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.common.collect.Lists;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.YearMonth;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

@Service
@Slf4j
@RequiredArgsConstructor
public class IncomeCompensationBizService {

    private final IncomeCompensationListService incomeCompensationListService;
    private final IncomeCompensationService incomeCompensationService;
    private final ApprovalRecordService approvalRecordService;
    private final KpiGroupUserBizService kpiGroupUserBizService;
    private final FlowApproveRpc flowApproveRpc;
    private final FlowVal flowVal;

    /**
     * 分页查询
     * @param dto
     * @return
     */
    public AppPage<IncomeCompensationVO> incomeCompensationPage(IncomeCompensationQueryDTO dto) {
        PageData<IncomeCompensation> pageData = incomeCompensationService.page(new PageData<>(dto.getCurrent(), dto.getPageSize()),
                Wrappers.<IncomeCompensation>lambdaQuery()
                        .eq(IncomeCompensation::getGroupId, dto.getGroupId())
                        .eq(PublicUtil.isNotEmpty(dto.getPostId()), IncomeCompensation::getPostId, dto.getPostId())
                        .eq(PublicUtil.isNotEmpty(dto.getStatus()), IncomeCompensation::getStatus, dto.getStatus())
                        .eq(IncomeCompensation::getYn, Boolean.TRUE)
                        .last("ORDER BY FIELD(`status`,3,1,2,4)")
        );
        return PublicUtil.toPage(pageData, incomeCompensation -> {
            IncomeCompensationVO incomeCompensationVO = IncomeCompensationVO.convertVO(incomeCompensation);
            return incomeCompensationVO;
        });
    }

    /**
     * 岗位最低收入补偿详情
     *
     * @param id
     * @return
     */
    public IncomeCompensationVO incomeCompensationDetail(Long id) {
        IncomeCompensation incomeCompensation = incomeCompensationService.getById(id);
        return PublicUtil.copy(incomeCompensation, IncomeCompensationVO.class);
    }

    /**
     * 保存
     *
     * @param dto
     * @param user
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public void saveIncomeCompensation(IncomeCompensationDTO dto, LoginAuthBean user) {
        checkIncomeCompensation(dto);
        IncomeCompensation incomeCompensation = convertDB(dto, user);
        incomeCompensationService.save(incomeCompensation);

        applyApproveCompensation(incomeCompensation, user);
    }

    /**
     * 提交审批
     *
     * @param compensation
     * @param user
     */
    @Transactional(rollbackFor = Exception.class)
    public void applyApproveCompensation(IncomeCompensation compensation, LoginAuthBean user){
        Date expTime = PublicUtil.getExpTimeOfMonth(LocalDate.now());
        Integer staffNum = kpiGroupUserBizService.queryShopPostCurStaffNum(compensation.getPostId(), compensation.getShopIds());
        //提交审批
        List<InitiateApprovalParam.Condition> conditions = Lists.newArrayList();
        InitiateApprovalParam.Condition postCondition = new InitiateApprovalParam.Condition();
        postCondition.setParamName("postId");
        postCondition.setValue(compensation.getPostId().toString());
        InitiateApprovalParam.Condition shopCondition = new InitiateApprovalParam.Condition();
        shopCondition.setParamName("shopId");
        List<String> shopIdsStr = compensation.getShopIds().stream().map(s -> s.toString()).collect(Collectors.toList());
        shopCondition.setValue(String.join(",", shopIdsStr));
        conditions.add(postCondition);
        conditions.add(shopCondition);
        Map extDataMap = new HashMap<String, Object>() {{
            put("id", compensation.getId().toString());
        }};
        FlowDto flowDto = FlowDto.builder()
                .groupId(compensation.getGroupId())
                .approvalNo(flowVal.getIncomeCompensationConfig())
                .briefContent(Lists.newArrayList(
                        "申请人:" + user.getUserName(),
                        "岗位:" + compensation.getPostName(),
                        "适用门店:" + compensation.getShopIds().size() + "个",
                        "适用人员:" + staffNum + "人",
                        "补偿月份:" + String.join(",", compensation.getMonths()),
                        "补偿金额:" + compensation.getMoney() + "元",
                        "申请时间:" + DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm").format(LocalDateTime.now())
                ))
                .initiatorId(user.getUserId())
                .expTime(expTime)
                .shopIds(compensation.getShopIds())
                .conditions(conditions)
                .extData(extDataMap)
                .build();
        String approvalNo = flowApproveRpc.initiate(flowDto);
        ApprovalRecord approvalRecord = ApprovalRecord.builder()
                .approvalNo(approvalNo)
                .approvalStatus(ApprovalStatusEnum.IN_APPROVAL)
                .approvalType(ApprovalTypeEnum.INCOME_COMPENSATION)
                .dataId(compensation.getId())
                .groupId(compensation.getGroupId())
                .build();
        approvalRecordService.save(approvalRecord);
    }


    /**
     * 检查绩效组配置状态
     *
     * @param dto
     */
    public void checkIncomeCompensation(IncomeCompensationDTO dto) {
        Long postId = dto.getPostId();
        List<Long> shopIds = dto.getShopIds();
//        List<IncomeCompensation> approves = incomeCompensationService.getRepeatConfigs(postId, shopIds, SettingStatusEnum.APPROVING);
//        if (PublicUtil.isNotEmpty(approves)) {
//            IncomeCompensation incomeCompensation = approves.get(0);
//            List<String> shopNames = new ArrayList<>(incomeCompensation.getShopNames());
//            shopNames.retainAll(dto.getShopNames());
//            throw new BusinessException("绩效组门店【" + String.join(",", shopNames) + "】存在" +
//                    incomeCompensation.getStatus().getName() + "配置");
//        }


        YearMonth curMonth = YearMonth.now();
        Optional<YearMonth> errorMonth = dto.getMonths().stream().filter(month -> month.isBefore(curMonth)).findAny();
        if (errorMonth.isPresent()) {
            throw new BusinessException("选择的月份只能是当月之后");
        }

    }

    /**
     * 转换对象
     *
     * @param dto
     * @param user
     * @return
     */
    public IncomeCompensation convertDB(IncomeCompensationDTO dto, LoginAuthBean user) {
        IncomeCompensation incomeCompensation = new IncomeCompensation();
        incomeCompensation.setPostId(dto.getPostId());
        incomeCompensation.setPostName(dto.getPostName());
        incomeCompensation.setShopIds(dto.getShopIds());
        incomeCompensation.setShopNames(dto.getShopNames());

        incomeCompensation.setMoney(dto.getMoney());
        incomeCompensation.setMaxMoney(dto.getMaxMoney());
        incomeCompensation.setStatus(SettingStatusEnum.APPROVING);
        incomeCompensation.setGroupId(user.getGroupId());
        incomeCompensation.setYn(Boolean.TRUE);

        List<String> yearMonths = dto.getMonths().stream().map(YearMonth::toString).collect(Collectors.toList());
        incomeCompensation.setMonths(yearMonths);
        return incomeCompensation;
    }


    /**
     * 岗位最低收入补偿
     *
     * @param approvalRecord
     * @param resultEvent
     */
    public void approvalCompensationConfig(ApprovalRecord approvalRecord, ApprovalResultEvent resultEvent) {
        log.info("收到岗位最低收入补偿审批信息：{}", JSON.toJSONString(resultEvent));
        Boolean agree = resultEvent.getAgree();
        Long compensationId = approvalRecord.getDataId();
        incomeCompensationService.update(Wrappers.<IncomeCompensation>lambdaUpdate()
                .set(IncomeCompensation::getStatus, (agree) ? SettingStatusEnum.EFFECTIVE : SettingStatusEnum.INEFFECTIVE)
                .eq(IncomeCompensation::getId, compensationId)
        );
        if (!resultEvent.getAgree()) {
            return;
        }
        IncomeCompensation compensation = incomeCompensationService.getById(approvalRecord.getDataId());

        List<IncomeCompensationList> effectCompensations = incomeCompensationListService.getRepeatConfigs(compensation.getPostId(),
                compensation.getShopIds(), compensation.getMonths());
        if (PublicUtil.isNotEmpty(effectCompensations)) {
            incomeCompensationListService.removeByIds(effectCompensations.stream().map(IncomeCompensationList::getId).collect(Collectors.toList()));
        }

        //转换为单条数据
        Long postId = compensation.getPostId();
        List<YearMonth> months = compensation.getMonths().stream().map(YearMonth::parse).collect(Collectors.toList());
        List<IncomeCompensationList> incomeCompensationLists = new ArrayList<>();
        for (Long shopId : compensation.getShopIds()) {
            for (YearMonth yearMonth : months) {
                IncomeCompensationList incomeCompensationList = IncomeCompensationList.builder()
                        .incomeCompensationId(compensationId)
                        .dataMonth(yearMonth)
                        .effective(Boolean.TRUE)
                        .postId(postId)
                        .shopId(shopId)
                        .ineffectiveDate(yearMonth.atEndOfMonth())
                        .yn(Boolean.TRUE)
                        .build();
                incomeCompensationLists.add(incomeCompensationList);
            }
        }

        incomeCompensationListService.saveBatch(incomeCompensationLists);




//        List<IncomeCompensation> effects = incomeCompensationService.getRepeatConfigs(compensation.getPostId(),
//                compensation.getShopIds(), SettingStatusEnum.EFFECTIVE);
//        List<Long> inEffectIds = effects.stream().filter(effect -> (! compensationId.equals(effect.getId())))
//                .map(IncomeCompensation::getId).collect(Collectors.toList());
//        if (PublicUtil.isEmpty(inEffectIds)) {
//            return;
//        }
//        log.info("岗位最低收入配置出现重复数据，{}", JSON.toJSONString(effects));
//        incomeCompensationService.update(Wrappers.<IncomeCompensation>lambdaUpdate()
//                .set(IncomeCompensation::getStatus, SettingStatusEnum.INEFFECTIVE)
//                .set(IncomeCompensation::getUpdateTime, new Date())
//                .in(IncomeCompensation::getId, inEffectIds)
//        );
    }
}
