package cn.fw.morax.service.biz.salary;

import cn.fw.morax.common.enums.FundType;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.common.utils.StringUtils;
import cn.fw.morax.domain.db.PayrollRecord;
import cn.fw.morax.domain.db.salary.SalaryConfirm;
import cn.fw.morax.domain.db.salary.SalaryGeneralSettin;
import cn.fw.morax.domain.db.salary.SalaryPool;
import cn.fw.morax.domain.enums.SalaryConfirmStatusEnum;
import cn.fw.morax.domain.enums.SalaryPayrollStatusEnum;
import cn.fw.morax.rpc.ehr.EhrRpcService;
import cn.fw.morax.rpc.ehr.dto.StaffBaseInfoDTO;
import cn.fw.morax.rpc.finance.FinanceRpcService;
import cn.fw.morax.rpc.finance.dto.FinanceParam;
import cn.fw.morax.rpc.finance.dto.FundParam;
import cn.fw.morax.rpc.oop.OopRpcService;
import cn.fw.morax.rpc.oop.dto.ShopDTO;
import cn.fw.morax.service.data.PayrollRecordService;
import cn.fw.morax.service.data.salary.SalaryConfirmService;
import cn.fw.morax.service.data.salary.SalaryGeneralSettinService;
import cn.fw.morax.service.data.salary.SalaryPoolService;
import cn.hutool.core.map.MapUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.BoundSetOperations;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.YearMonth;
import java.util.*;
import java.util.concurrent.CompletableFuture;

/**
 * 薪资发放业务类
 *
 * @author : kurisu
 * @version : 1.0
 * @className : PayoffBizService
 * @description : 薪资发放业务类
 * @date : 2022-05-07 17:10
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class PayoffBizService {
    /**
     * 财务业务类型
     */
    private final static Integer FINANCE_BUSINESS_TYPE = 20010;
    /**
     * 财务款项类型map
     */
    private final static Map<FundType, String> FUND_MAP = MapUtil.builder(new EnumMap<FundType, String>(FundType.class))
            .put(FundType.WAGE, "RP00000133")
            .put(FundType.AWARD, "RP00000113")
            .put(FundType.PENALTY, "RP00000098")
            .build();
    private final PayrollRecordService payrollRecordService;
    private final FinanceRpcService financeRpcService;
    private final OopRpcService oopRpcService;
    private final StringRedisTemplate stringRedisTemplate;
    private final SalaryPoolService salaryPoolService;
    private final SalaryGeneralSettinService salaryGeneralSettinService;
    private final SalaryConfirmService salaryConfirmService;
    private final EhrRpcService ehrRpcService;

    @Value("${spring.cache.custom.global-prefix}:salary:payoff")
    @Getter
    private String payoffKey;


    /**
     * 准备
     */
    public void read2payoff() {
        List<PayrollRecord> list = payrollRecordService.list(Wrappers.<PayrollRecord>lambdaQuery()
                .eq(PayrollRecord::getPayrollStatus, SalaryPayrollStatusEnum.WAITING)
        );
        if (CollectionUtils.isEmpty(list)) {
            return;
        }
        BoundSetOperations<String, String> setOps = stringRedisTemplate.boundSetOps(getPayoffKey());
        for (PayrollRecord payrollRecord : list) {
            String idstr = String.valueOf(payrollRecord.getId());
            if (!Boolean.TRUE.equals(setOps.isMember(idstr))) {
                setOps.add(idstr);
            }
        }
    }


    /**
     * 发放
     */
    public void go2payoff() {
        BoundSetOperations<String, String> setOps = stringRedisTemplate.boundSetOps(getPayoffKey());
        String str;
        while ((str = setOps.pop()) != null) {
            final PayrollRecord payrollRecord = payrollRecordService.getById(Long.valueOf(str));
            CompletableFuture.runAsync(() -> payoff(payrollRecord));
        }
    }

    /**
     * 生成工资发放记录
     *
     * @param groupId
     */
    public void createSalarySheet(Long groupId) {
        SalaryGeneralSettin settin = salaryGeneralSettinService.getOne(Wrappers.<SalaryGeneralSettin>lambdaQuery()
                        .eq(SalaryGeneralSettin::getYn, Boolean.TRUE)
                        .eq(SalaryGeneralSettin::getGroupId, groupId)
                , Boolean.FALSE);
        int payoffDate = Optional.ofNullable(settin.getPayoffDate()).orElse(15);
        int dayOfMonth = LocalDate.now().getDayOfMonth();
        if (dayOfMonth != payoffDate) {
            return;
        }
        List<SalaryPool> salaryPoolList = salaryPoolService.list(Wrappers.<SalaryPool>lambdaQuery()
                .eq(SalaryPool::getMonthly, YearMonth.now().minusMonths(1))
                .eq(SalaryPool::getPaid, Boolean.FALSE)
                .eq(SalaryPool::getGroupId, groupId)
                .eq(SalaryPool::getYn, Boolean.TRUE)
        );
        if (CollectionUtils.isEmpty(salaryPoolList)) {
            return;
        }
        for (SalaryPool salaryPool : salaryPoolList) {
            try {
                autoConfirmTodoNotice(salaryPool.getId());
                createSalarySheet(salaryPool, BigDecimal.valueOf(settin.getPayoffLimit()));
                salaryPool.setRegular(Boolean.TRUE);
                salaryPool.setPaid(Boolean.TRUE);
            } catch (Exception e) {
                log.error("生产工资发放记录失败 -> pooId: [{}]", salaryPool.getId(), e);
            }
        }
        salaryPoolService.updateBatchById(salaryPoolList);
    }


    /**
     * 生产工资发放记录
     *
     * @param pool
     */
    @Transactional(rollbackFor = Exception.class)
    public void createSalarySheet(SalaryPool pool, BigDecimal maxAmount) {
        List<PayrollRecord> list = new ArrayList<>();
        PayrollRecord corporateRecord = createByPool(pool);
        if (Objects.isNull(corporateRecord)) {
            return;
        }
        // fixme 暂时自动发放工资不对接财务，后续放开
        corporateRecord.setPayrollStatus(SalaryPayrollStatusEnum.COMPLETE);
        BigDecimal reward = pool.getReward();
        if (maxAmount.compareTo(BigDecimal.ZERO) > 0 && reward.compareTo(maxAmount) > 0) {
            corporateRecord.setAmount(maxAmount);
            list.add(corporateRecord);
            PayrollRecord record = new PayrollRecord();
            BeanUtils.copyProperties(corporateRecord, record);
            record.setCorporate(Boolean.FALSE);
            record.setAmount(reward.subtract(maxAmount));
            list.add(record);
        } else {
            list.add(corporateRecord);
        }
        payrollRecordService.saveBatch(list);
    }

    /**
     * 自动确认待办通知
     *
     * @param poolId
     */
    @Transactional(rollbackFor = Exception.class)
    public void autoConfirmTodoNotice(Long poolId) {
        SalaryConfirm salaryConfirm = salaryConfirmService.getOne(Wrappers.<SalaryConfirm>lambdaQuery()
                        .eq(SalaryConfirm::getSalaryPoolId, poolId)
                        .eq(SalaryConfirm::getConfirmedStatus, SalaryConfirmStatusEnum.NEED_CONFIRM)
                        .eq(SalaryConfirm::getYn, Boolean.TRUE)
                , Boolean.FALSE);
        if (Objects.isNull(salaryConfirm)) {
            return;
        }
        salaryConfirm.setConfirmedStatus(SalaryConfirmStatusEnum.CONFIRMED);
        salaryConfirm.setSelfConfirm(Boolean.FALSE);
        salaryConfirm.setConfirmTime(salaryConfirm.getDeadline());
        salaryConfirmService.updateById(salaryConfirm);
    }


    @Transactional(rollbackFor = Exception.class)
    public void payoff(final PayrollRecord payrollRecord) {
        if (Objects.isNull(payrollRecord)) {
            return;
        }
        if (!SalaryPayrollStatusEnum.WAITING.equals(payrollRecord.getPayrollStatus())) {
            return;
        }
        FinanceParam financeParam = transferFinanceParam(payrollRecord);
        if (Objects.isNull(financeParam)) {
            return;
        }
        String token;
        if (financeParam.isPay()) {
            token = financeRpcService.paySomething(financeParam);
        } else {
            token = financeRpcService.deductSomething(financeParam);
        }
        if (StringUtils.isEmpty(token)) {
            return;
        }
        payrollRecord.setPayrollStatus(SalaryPayrollStatusEnum.SEND);
        payrollRecord.setPayrollToken(token);
        payrollRecordService.updateById(payrollRecord);
    }

    private FinanceParam transferFinanceParam(final PayrollRecord payrollRecord) {
        Long dealerId = payrollRecord.getDealerId();
        Long shopId = PublicUtil.isNotEmpty(payrollRecord.getFundShopId()) ?
                payrollRecord.getFundShopId() : payrollRecord.getShopId();
        if (Objects.isNull(dealerId)) {
            ShopDTO shopDTO = oopRpcService.queryShop(shopId);
            if (Objects.isNull(shopDTO)) {
                return null;
            }
            dealerId = shopDTO.getDealerId();
        }
        FinanceParam fp = new FinanceParam();
        fp.setBusinessType(FINANCE_BUSINESS_TYPE);
        fp.setBusinessNo(String.valueOf(payrollRecord.getId()));
        fp.setShopId(shopId);
        fp.setDealerId(dealerId);
        fp.setGroupId(payrollRecord.getGroupId());

        FundParam fundParam = new FundParam();
        fundParam.setUserId(payrollRecord.getUserId());
        fundParam.setUserName(payrollRecord.getUserName());
        fundParam.setCorporate(Boolean.TRUE.equals(payrollRecord.getCorporate()));
        String type;
        if (Boolean.TRUE.equals(payrollRecord.getPayment())) {
            type = FUND_MAP.get(FundType.AWARD);
            fp.setPay(Boolean.TRUE);
        } else {
            type = FUND_MAP.get(FundType.PENALTY);
            fp.setPay(Boolean.FALSE);
        }
        if (Boolean.TRUE.equals(payrollRecord.getSalary())) {
            type = FUND_MAP.get(FundType.WAGE);
            fp.setPay(Boolean.TRUE);
        }
        fundParam.setType(type);
        fundParam.setAmount(payrollRecord.getAmount());
        StaffBaseInfoDTO staffBaseInfoDTO = ehrRpcService.queryStaffBaseInfo(payrollRecord.getUserId());
        if (PublicUtil.isNotEmpty(staffBaseInfoDTO) && PublicUtil.isNotEmpty(staffBaseInfoDTO.getMobile())) {
            fundParam.setMobile(staffBaseInfoDTO.getMobile());
        }
        fp.setFundParam(fundParam);
        return fp;
    }

    private PayrollRecord createByPool(SalaryPool pool) {
        if (Objects.isNull(pool)) {
            return null;
        }
        PayrollRecord record = new PayrollRecord();
        Long fundShopId = PublicUtil.isNotEmpty(pool.getFundShopId()) ?
                pool.getFundShopId() : pool.getShopId();
        record.setFundShopId(fundShopId);
        ShopDTO shopDTO = oopRpcService.queryShop(fundShopId);
        if (Objects.nonNull(shopDTO)) {
            record.setDealerId(shopDTO.getDealerId());
        }
        record.setShopId(pool.getShopId());
        record.setUserId(pool.getUserId());
        record.setUserName(pool.getUserName());
        record.setBizId(String.valueOf(pool.getId()));
        record.setPayment(Boolean.TRUE);
        record.setSalary(Boolean.TRUE);
        record.setAmount(pool.getReward());
        record.setCorporate(Boolean.TRUE);
        record.setPayrollStatus(SalaryPayrollStatusEnum.WAITING);
        record.setMonthly(pool.getMonthly());
        record.setGroupId(pool.getGroupId());
        return record;
    }

}
