package cn.fw.morax.service.biz.salary;

import cn.fw.common.exception.BusinessException;
import cn.fw.common.web.annotation.DisLock;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.db.salary.SalaryGeneralSettin;
import cn.fw.morax.domain.dto.salary.SalaryGeneralSettinDTO;
import cn.fw.morax.domain.vo.salary.SalaryGeneralSettinVO;
import cn.fw.morax.service.data.salary.SalaryGeneralSettinService;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDate;
import java.time.ZoneId;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Optional;

import static cn.fw.common.businessvalidator.Validator.BV;

@RequiredArgsConstructor
@Service
@Slf4j
public class SalaryGeneralSettinBizService {

    private final SalaryGeneralSettinService salaryGeneralSettinService;

    @Value("${spring.cache.custom.global-prefix}:salary:system:save:")
    @Getter
    private String savePrefix;

    /**
     * 查询薪酬系统配置
     *
     * @param groupId
     * @return
     */
    public SalaryGeneralSettinVO getSystemConfig(Long groupId) {
        SalaryGeneralSettin salaryGeneralSettin = salaryGeneralSettinService.getOne(Wrappers.<SalaryGeneralSettin>lambdaUpdate()
                .eq(SalaryGeneralSettin::getGroupId, groupId)
        );
        if (PublicUtil.isEmpty(salaryGeneralSettin)) {
            return new SalaryGeneralSettinVO();
        }
        SalaryGeneralSettinVO settingVo = PublicUtil.copy(salaryGeneralSettin, SalaryGeneralSettinVO.class);

        String[] datesOfAppeal = salaryGeneralSettin.getDatesOfAppeal().split(",");
        settingVo.setStartDateOfAppeal(datesOfAppeal[0]);
        settingVo.setEndDateOfAppeal(datesOfAppeal[1]);
        return settingVo;
    }


    /**
     * 薪酬系统配置保存
     *
     * @param dto
     */
    @Transactional(rollbackFor = Exception.class)
    @DisLock(prefix = "#this.getSavePrefix()", key = "#dto.getGroupId()", message = "保存中，请勿重复操作")
    public void systemConfig(SalaryGeneralSettinDTO dto) {
        this.checkSystemConfig(dto);
        salaryGeneralSettinService.remove(Wrappers.<SalaryGeneralSettin>lambdaUpdate()
                .eq(SalaryGeneralSettin::getGroupId, dto.getGroupId())
        );
        SalaryGeneralSettin setting = PublicUtil.copy(dto, SalaryGeneralSettin.class);
        List<String> datesOfAppeal = new ArrayList<String>() {{
            add(dto.getStartDateOfAppeal().toString());
            add(dto.getEndDateOfAppeal().toString());
        }};
        setting.setPayoffLimit(Optional.ofNullable(dto.getPayoffLimit()).orElse(0));
        setting.setDatesOfAppeal(String.join(",", datesOfAppeal));
        salaryGeneralSettinService.save(setting);
    }

    /**
     * 检查配置
     *
     * @param dto
     */
    public void checkSystemConfig(SalaryGeneralSettinDTO dto) {
        BV.isTrue((dto.getStartDateOfAppeal() < dto.getEndDateOfAppeal()), "可申诉开始日期不能在可申诉结束日期之后");
        BV.isTrue((dto.getPayoffDate() > dto.getEndDateOfAppeal()), "工资发放时间日期只能在可申诉结束日期之后");
    }

    /**
     * 查询薪酬系统配置
     *
     * @param groupId
     * @return
     */
    public Date checkAppealDate(Long groupId) {
        SalaryGeneralSettin salaryGeneralSettin = salaryGeneralSettinService.getOne(Wrappers.<SalaryGeneralSettin>lambdaUpdate()
                .eq(SalaryGeneralSettin::getGroupId, groupId)
        );
        BV.notNull(salaryGeneralSettin, "请联系管理人员配置薪酬系统配置");
        LocalDate now = LocalDate.now();
        BV.isTrue(salaryGeneralSettin.getDatesOfAppeal().contains(","), "薪酬可申诉日期系统配置有误");
        String[] datesOfAppeal = salaryGeneralSettin.getDatesOfAppeal().split(",");
        Integer startDateOfAppeal = Integer.parseInt(datesOfAppeal[0]);
        Integer endDateOfAppeal = Integer.parseInt(datesOfAppeal[1]);
        if ((now.getDayOfMonth() < startDateOfAppeal) || (now.getDayOfMonth() > endDateOfAppeal)) {
            throw new BusinessException("工资申诉时间在每月：" + String.join("-", datesOfAppeal) + "之间，当前不能申请");
        }
        LocalDate expTime = now.withDayOfMonth(endDateOfAppeal);
        return Date.from(expTime.atStartOfDay().atZone(ZoneId.systemDefault()).toInstant());
    }

    /**
     * 获取审批过期时间
     *
     * @param groupId
     * @return
     */
    public Date getSalaryAppealExpTime(Long groupId) {
        SalaryGeneralSettin salaryGeneralSettin = salaryGeneralSettinService.getOne(Wrappers.<SalaryGeneralSettin>lambdaUpdate()
                .eq(SalaryGeneralSettin::getGroupId, groupId)
        );
        BV.notNull(salaryGeneralSettin, "请联系管理人员配置薪酬系统配置");
        LocalDate now = LocalDate.now();
        String[] datesOfAppeal = salaryGeneralSettin.getDatesOfAppeal().split(",");
        Integer endDateOfAppeal = Integer.parseInt(datesOfAppeal[1]);
        LocalDate expTime = now.withDayOfMonth(endDateOfAppeal);
        return Date.from(expTime.atStartOfDay().atZone(ZoneId.systemDefault()).toInstant());
    }

}
